#include <REG51.H>

#include "lcd_low_level_4bit.h"
#include "wait.h"


/******************** lcd_port() *****************/
unsigned char lcd_port(unsigned char val) reentrant{	/* Send 4 bits */
	LCD_PORT = val | 4;	// E 1->0 Clock
	LCD_PORT = val;	
	return 0;	// OK
}

/******************** lcd_busy() *****************/
void lcd_busy(void) reentrant{
	unsigned int i;
	unsigned char busy;

	LCD_PORT = 0xF2;
	for(i=0;i<2000;i++){	// about max 20 msec on a 20MHz machine...
		LCD_PORT = 0xF6;	// RW=1;
		busy=LCD_PORT&0x80;	// Read High-Nibble
		LCD_PORT = 0xF2;
		if(!busy) break;	
		LCD_PORT = 0xF6;	// Read Low-Nibble (Dummy)
		LCD_PORT = 0xF2;
	}
	LCD_PORT = 0xF0;
	return;		// Return even if there is NO Display after a certain time...
}

/******************** lcd_databyte(): Data-Byte (RS=1) *************/
void lcd_databyte(unsigned char date) reentrant {

	lcd_busy();
	lcd_port((date&0xF0)|1);	/* Highbyte, RS=1 */
	lcd_port(((date<<4))|1);	/* Lowbyte, RS=1 */


}

/******************** lcd_combyte(): Command-Byte (RS=0) *************/
void lcd_combyte(unsigned char date) reentrant {

	lcd_busy();
	lcd_port((date&0xF0));	/* Highbyte, RS=0 */
	lcd_port(((date<<4)));	/* Lowbyte, RS=0 */


}

/***************** lcd_initport(): Initialize Port, Optionally reset LCD and setz Access-Type **********/
unsigned char lcd_initport(void){

	lcd_port(255);
	wait_ms(200);		/* Wait abt. 200 ms for RESET */
	lcd_port(247);
	wait_ms(200);		/* Wait abt. 200 ms for POWER ON */

	lcd_port(0x30);
	wait_ms(5);
	lcd_port(0x30);
	wait_ms(1);
	lcd_port(0x30);
	wait_ms(1);
	lcd_port(0x20);		/* Switch to 4 Bit... */
	wait_ms(1);
	lcd_port(0x20);		/* Now: 4 Bit, Small Font, >=2 lines */
	wait_ms(1);
	lcd_port(0x80);
	wait_ms(1);
	return 0;
}